package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.bank.to.BankTo;
import gz.cyberbrain.cyberhotel.model.card.to.CardTo;
import gz.cyberbrain.cyberhotel.model.client.to.AgencyTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientPayerException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientWithInvoicesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidPayModeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.AgencyWithReservationsException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAddressException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableBankException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCardException;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.ImageItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class AgencyController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate delegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();
	
	private AgencyController() {}
	
	public static Vector agencyList() throws InternalErrorException {				
		Vector<TextItem> items = new Vector<TextItem>();
		Collection agencies = delegate.showAgencies();
		Iterator iter = agencies.iterator();

		AgencyTo agency;

		for (int i=0; iter.hasNext(); i++) {

			agency = (AgencyTo) iter.next();
			items.add(new TextItem(agency.getClientId(), agency.getName()));

		}

		return items;
	}

	public static Serializable createAgency(Long address, String street, String dwellingPlace, 
			String phone1, String phone2, String fax, String email, String web, Long bank, Long card, String notes, 
			String cif, String name, String contact, String payMode, String accountNumber, 
			String cardNumber) throws InvalidPayModeException, UnaceptableAddressException, 
									  UnaceptableBankException, UnaceptableCardException, InternalErrorException {
	
		return delegate.insertAnAgency(new AgencyTo(null, address, street, dwellingPlace, phone1, phone2, fax, 
				email, web, bank, card, notes, cif, name, contact, payMode, accountNumber, cardNumber)).getClientId();
		
	}

	public static void updateAgency(Long clientId, Long address, String street, String dwellingPlace, 
			String phone1, String phone2, String fax, String email, String web, Long bank, Long card, String notes, 
			String cif, String name, String contact, String payMode, String accountNumber, 
			String cardNumber) throws InvalidPayModeException, UnaceptableAddressException, UnaceptableBankException,
									  UnaceptableCardException, InstanceNotFoundException, InternalErrorException{				
		
		delegate.updateAnAgency(new AgencyTo(clientId, address, street, dwellingPlace, phone1, phone2, fax, 
				email, web, bank, card, notes, cif, name, contact, payMode, accountNumber, cardNumber));
		
	}
	
	public static void deleteAgency(Long clientId) 
		throws InstanceNotFoundException, InternalErrorException,  
		AgencyWithReservationsException, ClientPayerException, ClientWithInvoicesException {
		
		AgencyTo agency = delegate.findAnAgency(clientId);
		delegate.removeAnAgency(agency);
		
	}
	
	public static HashMap findAgency(Long clientId) throws InstanceNotFoundException, InternalErrorException {
		
		AgencyTo agency = delegate.findAnAgency(clientId);
		
		attributes.put("id", clientId);
		attributes.put("address", agency.getAddress());
		attributes.put("street", agency.getStreet());
		attributes.put("dwellingPlace", agency.getDwellingPlace());
		attributes.put("phone1", agency.getPhone1());
		attributes.put("phone2", agency.getPhone2());
		attributes.put("fax", agency.getFax());
		attributes.put("email", agency.getEmail());
		attributes.put("web", agency.getWeb());
		
		BankTo bank = config.findBank(agency.getBank());
		attributes.put("bank", new ImageItem(bank.getBankId(), bank.getName(), bank.getLogo()));
		
		CardTo card = config.findCard(agency.getCard());
		attributes.put("card", new ImageItem(card.getCardId(), card.getType(), card.getLogo()));
		
		attributes.put("notes", agency.getNotes());
		attributes.put("cif", agency.getCif());
		attributes.put("name", agency.getName());
		attributes.put("contact", agency.getContact());
		attributes.put("payMode", agency.getPayMode());
		attributes.put("accountNumber", agency.getAccountNumber());
		attributes.put("cardNumber", agency.getCardNumber());		
				
		return attributes;
				
	}

	
}
