package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.controller.util.ErrorItem;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.RoomWithBoardersException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableRoomTypeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomBusyException;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.roomtype.to.RoomTypeTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class RoomController {

	private static ConfigurationFacadeDelegate confDelegate = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate resDelegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();

	private RoomController() {}

	public static Vector roomList() {				
		Vector<TextItem> items = new Vector<TextItem>();
		Collection rooms;
		try {
			rooms = confDelegate.showRooms();
	
		Iterator iter = rooms.iterator();

		RoomTo room;

		for (int i=0; iter.hasNext(); i++) {

			room = (RoomTo) iter.next();
			items.add(new TextItem(room.getRoomCode(), room.getNumber()));

		}

		} catch (InternalErrorException e) {
			e.printStackTrace();
		}
		
		return items;
	}

	public static Serializable createRoom(String roomCode, String number, long floor, String type, boolean smokers, long maxPax, String notes) throws UnaceptableRoomTypeException, InternalErrorException {
		return confDelegate.createRoom(new RoomTo(roomCode, number, floor, type, "Libre", "Limpa", Calendar.getInstance(), smokers, maxPax, notes));		
	}

	public static void updateRoom(String roomCode, String number, long floor, String type, boolean smokers, long maxPax, String notes) throws UnaceptableRoomTypeException, InstanceNotFoundException, InternalErrorException {		
		RoomTo room = confDelegate.findRoom(roomCode);
		confDelegate.updateRoom(new RoomTo(roomCode, number, floor, type, room.getOccupancyStatus(), room.getCleaningStatus(), room.getCheckingDate(), smokers, maxPax, notes));
	}

	public static void deleteRoom(String roomCode) throws InstanceNotFoundException, RoomWithBoardersException, InternalErrorException{
		RoomTo room = confDelegate.findRoom(roomCode);
		confDelegate.deleteRoom(room);
	}

	public static HashMap findRoom(String roomCode) throws InstanceNotFoundException, InternalErrorException {

		RoomTo room = confDelegate.findRoom(roomCode);

		attributes.put("code", roomCode);
		attributes.put("number", room.getNumber());
		attributes.put("floor", room.getFloor());

		RoomTypeTo roomType = confDelegate.findRoomType(room.getType());

		attributes.put("type", new TextItem(roomType.getRoomTypeCode(), roomType.getDescription()));
		attributes.put("occupancyStatus", room.getOccupancyStatus());
		attributes.put("cleaningStatus", room.getCleaningStatus());
		attributes.put("checkingDate", room.getCheckingDate());
		attributes.put("smokers", room.getSmokers());
		attributes.put("maxPax", room.getMaxPax());
		attributes.put("notes", room.getNotes());

		return attributes;

	}

	public static void lock(String roomCode) {

		try {

			RoomTo room = confDelegate.findRoom(roomCode);

			resDelegate.blockARoom(room);

		} catch (RoomBusyException e) {
			ErrorsHandler.add(new ErrorItem("roomBusy"));			
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

	}

	public static void unlock(String roomCode) {

		try {

			RoomTo room = confDelegate.findRoom(roomCode);

			resDelegate.unblockARoom(room);

		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		}

	}

}
