package gz.cyberbrain.cyberhotel.model.configurationfacade.spring;

import java.util.Calendar;
import java.util.Collection;

import gz.cyberbrain.cyberhotel.model.address.dao.AddressDao;
import gz.cyberbrain.cyberhotel.model.address.to.AddressTo;
import gz.cyberbrain.cyberhotel.model.bank.dao.BankDao;
import gz.cyberbrain.cyberhotel.model.bank.to.BankTo;
import gz.cyberbrain.cyberhotel.model.board.dao.BoardDao;
import gz.cyberbrain.cyberhotel.model.board.to.BoardTo;
import gz.cyberbrain.cyberhotel.model.card.dao.CardDao;
import gz.cyberbrain.cyberhotel.model.card.to.CardTo;
import gz.cyberbrain.cyberhotel.model.concept.dao.ConceptDao;
import gz.cyberbrain.cyberhotel.model.concept.to.ConceptTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.AddressInUseException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.BankWithClientsException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.BoardInUseException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.BoardTariffiedException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.CardWithClientsException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.ConceptInUseException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.EmployeeWithWashesException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.PriceTypeInUseException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.PriceTypeTariffiedException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.RoomTypeInUseException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.RoomWithBoardersException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.SeasonTariffiedException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableRoomTypeException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UserInUseException;
import gz.cyberbrain.cyberhotel.model.owe.dao.OweDao;
import gz.cyberbrain.cyberhotel.model.price.dao.PriceDao;
import gz.cyberbrain.cyberhotel.model.price.to.PriceTo;
import gz.cyberbrain.cyberhotel.model.pricetype.dao.PriceTypeDao;
import gz.cyberbrain.cyberhotel.model.pricetype.to.PriceTypeTo;
import gz.cyberbrain.cyberhotel.model.request.dao.RequestDao;
import gz.cyberbrain.cyberhotel.model.role.dao.RoleDao;
import gz.cyberbrain.cyberhotel.model.room.dao.RoomDao;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.roomtype.dao.RoomTypeDao;
import gz.cyberbrain.cyberhotel.model.roomtype.to.RoomTypeTo;
import gz.cyberbrain.cyberhotel.model.season.dao.SeasonDao;
import gz.cyberbrain.cyberhotel.model.season.to.SeasonTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.spring.actions.*;
import gz.cyberbrain.cyberhotel.model.day.dao.DayDao;
import gz.cyberbrain.cyberhotel.model.day.to.DayTo;
import gz.cyberbrain.cyberhotel.model.employee.dao.EmployeeDao;
import gz.cyberbrain.cyberhotel.model.employee.to.EmployeeTo;
import gz.cyberbrain.cyberhotel.model.user.dao.UserDao;
import gz.cyberbrain.cyberhotel.model.user.to.UserTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

public class ConfigurationSpringFacadeDelegate implements ConfigurationFacadeDelegate {

	private static final long serialVersionUID = 625492978683808444L;

	private AddressDao addressDao;
	private BankDao bankDao;
	private CardDao cardDao;
	private RoomTypeDao roomTypeDao;
	private RoomDao roomDao;
	private BoardDao boardDao;
	private PriceTypeDao priceTypeDao;
	private SeasonDao seasonDao;
	private PriceDao priceDao;
	private DayDao dayDao;
	private ConceptDao conceptDao;
	private EmployeeDao employeeDao;
	private UserDao userDao;
	private RoleDao roleDao;
	private OweDao oweDao;
	private RequestDao requestDao;

	public void setAddressDao(AddressDao addressDao) {
		this.addressDao = addressDao;
	}

	public void setBankDao(BankDao bankDao) {
		this.bankDao = bankDao;
	}

	public void setCardDao(CardDao cardDao) {
		this.cardDao = cardDao;
	}

	public void setRoomTypeDao(RoomTypeDao roomTypeDao) {		
		this.roomTypeDao = roomTypeDao;
	}
	public void setRoomDao(RoomDao roomDao) {
		this.roomDao = roomDao;
	}
	public void setBoardDao(BoardDao boardDao) {
		this.boardDao = boardDao;
	}
	public void setPriceTypeDao(PriceTypeDao priceTypeDao) {
		this.priceTypeDao = priceTypeDao;
	}
	public void setSeasonDao(SeasonDao seasonDao) {
		this.seasonDao = seasonDao;
	}
	public void setPriceDao(PriceDao priceDao) {
		this.priceDao = priceDao;
	}
	public void setDayDao(DayDao dayDao) {
		this.dayDao = dayDao;
	}
	public void setConceptDao(ConceptDao conceptDao) {
		this.conceptDao = conceptDao;
	}

	public void setEmployeeDao(EmployeeDao employeeDao) {
		this.employeeDao = employeeDao;
	}
	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	public void setRoleDao(RoleDao roleDao) {
		this.roleDao = roleDao;
	}

	public void setOweDao(OweDao oweDao) {
		this.oweDao = oweDao;
	}

	public void setRequestDao(RequestDao requestDao) {
		this.requestDao = requestDao;
	}

	public AddressTo createAddress(AddressTo addressTo) throws InternalErrorException {

		try{

			//AddressDao addressDao = SpringDaoFactory.getAddressDao();
			return addressDao.create(addressTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public AddressTo findAddress(Long addressId) throws InstanceNotFoundException, InternalErrorException {

		try {

			//AddressDao addressDao = SpringDaoFactory.getAddressDao();
			return addressDao.find(addressId);

		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void updateAddress(AddressTo addressTo) throws InternalErrorException {

		try{

			//AddressDao addressDao = SpringDaoFactory.getAddressDao();
			addressDao.update(addressTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteAddress(AddressTo addressTo) throws AddressInUseException, InternalErrorException {

		try {

			DeleteAddressAction action = new DeleteAddressAction(addressTo, addressDao);
			action.execute();

		}catch (AddressInUseException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showAddresses() throws InternalErrorException {

		try{


			//AddressDao addressDao = SpringDaoFactory.getAddressDao();			
			return addressDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public Collection showCountries() throws InternalErrorException {

		try{


			//AddressDao addressDao = SpringDaoFactory.getAddressDao();			
			return addressDao.findCountries();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showCountryProvinces(String country) throws InternalErrorException {

		try{


			//AddressDao addressDao = SpringDaoFactory.getAddressDao();			
			return addressDao.findProvincesByCountry(country);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public Collection showProvinceLocalities(String country, String province) throws InternalErrorException {

		try{


			//AddressDao addressDao = SpringDaoFactory.getAddressDao();			
			return addressDao.findLocalitiesByProvince(country, province);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public AddressTo findByCompleteAddress(String country, String province, String locality) 
	throws InstanceNotFoundException, InternalErrorException {

		try {

			//AddressDao addressDao = SpringDaoFactory.getAddressDao();
			return addressDao.findByCompleteAddress(country, province, locality);

		} catch (InstanceNotFoundException e) {			
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteAddressByCountry(String country) throws AddressInUseException, InternalErrorException {

		try{

			DeleteAddressByCountryAction action = new DeleteAddressByCountryAction(country, addressDao);
			action.execute();

		} catch (AddressInUseException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteAddressByProvince(String country, String province) throws AddressInUseException, InternalErrorException {

		try{

			DeleteAddressByProvinceAction action = new DeleteAddressByProvinceAction(country, province, addressDao);
			action.execute();

		} catch (AddressInUseException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteAddressByLocality(String country, String province, String locality) throws AddressInUseException, InternalErrorException {

		try{

			DeleteAddressByLocalityAction action = new DeleteAddressByLocalityAction(country, province, locality, addressDao);
			action.execute();

		} catch (AddressInUseException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public BankTo createBank(BankTo bankTo) throws InternalErrorException {

		try{

			//BankDao bankDao = SpringDaoFactory.getBankDao();			
			return bankDao.create(bankTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public BankTo findBank(Long bankId) throws InstanceNotFoundException, InternalErrorException {

		try {

			//BankDao bankDao = SpringDaoFactory.getBankDao();
			return bankDao.find(bankId);

		} catch (InstanceNotFoundException e) {			
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void updateBank(BankTo bankTo) throws InternalErrorException {

		try{

			//BankDao bankDao = SpringDaoFactory.getBankDao();
			bankDao.update(bankTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteBank(BankTo bankTo) throws BankWithClientsException, InternalErrorException {

		try{

			DeleteBankAction action = new DeleteBankAction(bankTo, bankDao);
			action.execute();

		}catch (BankWithClientsException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showBanks() throws InternalErrorException {

		try{


			//BankDao bankDao = SpringDaoFactory.getBankDao();			
			return bankDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public CardTo createCard(CardTo cardTo) throws InternalErrorException {

		try{

			//CardDao cardDao = SpringDaoFactory.getCardDao();			
			return cardDao.create(cardTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public CardTo findCard(Long cardId) throws InstanceNotFoundException, InternalErrorException {

		try {

			//CardDao cardDao = SpringDaoFactory.getCardDao();
			return cardDao.find(cardId);

		} catch (InstanceNotFoundException e) {			
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void updateCard(CardTo cardTo) throws InternalErrorException {

		try{

			//CardDao cardDao = SpringDaoFactory.getCardDao();
			cardDao.update(cardTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteCard(CardTo cardTo) throws CardWithClientsException, InternalErrorException {

		try{

			DeleteCardAction action = new DeleteCardAction(cardTo, cardDao);
			action.execute();

		}catch (CardWithClientsException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showCards() throws InternalErrorException {

		try{

			//CardDao cardDao = SpringDaoFactory.getCardDao();			
			return cardDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public RoomTypeTo createRoomType(RoomTypeTo roomTypeTo) throws InternalErrorException {

		try{

			//RoomTypeDao roomTypeDao = SpringDaoFactory.getRoomTypeDao();
			return roomTypeDao.create(roomTypeTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public RoomTypeTo findRoomType(String roomTypeCode) throws InstanceNotFoundException, InternalErrorException {

		try {

			//RoomTypeDao roomTypeDao = SpringDaoFactory.getRoomTypeDao();
			return roomTypeDao.find(roomTypeCode); 
		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public void updateRoomType(RoomTypeTo roomTypeTo) throws InternalErrorException {

		try{

			//RoomTypeDao roomTypeDao = SpringDaoFactory.getRoomTypeDao();
			roomTypeDao.update(roomTypeTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteRoomType(RoomTypeTo roomTypeTo) throws RoomTypeInUseException, InternalErrorException {

		try{

			DeleteRoomTypeAction action = new DeleteRoomTypeAction(roomTypeTo, roomTypeDao);
			action.execute();

		}catch (RoomTypeInUseException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showRoomTypes() throws InternalErrorException {

		try{

			//RoomTypeDao roomTypeDao = SpringDaoFactory.getRoomTypeDao();			
			return roomTypeDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public RoomTo createRoom(RoomTo roomTo) throws UnaceptableRoomTypeException, InternalErrorException {

		try{

			CreateRoomAction action = new CreateRoomAction(roomTo, roomTypeDao, roomDao);			
			return action.execute();

		} catch (UnaceptableRoomTypeException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public RoomTo findRoom(String roomCode) throws InstanceNotFoundException, InternalErrorException {

		try {

			//RoomDao roomDao = SpringDaoFactory.getRoomDao();
			return roomDao.find(roomCode);

		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}


	public void updateRoom(RoomTo roomTo) throws UnaceptableRoomTypeException, InternalErrorException {

		try{

			UpdateRoomAction action = new UpdateRoomAction(roomTo, roomTypeDao, roomDao);			
			action.execute();

		} catch (UnaceptableRoomTypeException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteRoom(RoomTo roomTo) throws RoomWithBoardersException, InternalErrorException {

		try{

			DeleteRoomAction action = new DeleteRoomAction(roomTo, roomDao);
			action.execute();

		}catch (InternalErrorException e) {
			throw e;
		}catch (RoomWithBoardersException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showRooms() throws InternalErrorException {

		try{

			//RoomDao roomDao = SpringDaoFactory.getRoomDao();			
			return roomDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public BoardTo createBoard(BoardTo boardTo) throws InternalErrorException {

		try{

			//BoardDao boardDao = SpringDaoFactory.getBoardDao();
			return boardDao.create(boardTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public BoardTo findBoard(String boardCode) throws InstanceNotFoundException, InternalErrorException {

		try {

			//BoardDao boardDao = SpringDaoFactory.getBoardDao();
			return boardDao.find(boardCode);

		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public void updateBoard(BoardTo boardTo) throws InternalErrorException {

		try{

			//BoardDao boardDao = SpringDaoFactory.getBoardDao();
			boardDao.update(boardTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteBoard(BoardTo boardTo) throws BoardInUseException, BoardTariffiedException, InternalErrorException {

		try{

			DeleteBoardAction action = new DeleteBoardAction(boardTo, boardDao);
			action.execute();

		}catch (BoardInUseException e){
			throw e;
		}catch (BoardTariffiedException e){
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showBoards() throws InternalErrorException {

		try{


			//BoardDao boardDao = SpringDaoFactory.getBoardDao();			
			return boardDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public PriceTypeTo createPriceType(PriceTypeTo priceTypeTo) throws InternalErrorException {

		try{

			//PriceTypeDao priceTypeDao = SpringDaoFactory.getPriceTypeDao();
			return priceTypeDao.create(priceTypeTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public PriceTypeTo findPriceType(String priceTypeCode) throws InstanceNotFoundException, InternalErrorException {

		try {

			//PriceTypeDao priceTypeDao = SpringDaoFactory.getPriceTypeDao();
			return priceTypeDao.find(priceTypeCode);

		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}


	public void updatePriceType(PriceTypeTo priceTypeTo) throws InternalErrorException {

		try{

			//PriceTypeDao priceTypeDao = SpringDaoFactory.getPriceTypeDao();
			priceTypeDao.update(priceTypeTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deletePriceType(PriceTypeTo priceTypeTo) throws PriceTypeInUseException, PriceTypeTariffiedException, InternalErrorException {

		try{

			DeletePriceTypeAction action = new DeletePriceTypeAction(priceTypeTo, priceTypeDao);
			action.execute();

		}catch (PriceTypeInUseException e){
			throw e;
		}catch (PriceTypeTariffiedException e){
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showPriceTypes() throws InternalErrorException {

		try{

			//PriceTypeDao priceTypeDao = SpringDaoFactory.getPriceTypeDao();		
			return priceTypeDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public SeasonTo createSeason(SeasonTo seasonTo) throws InternalErrorException {

		try{

			//SeasonDao seasonDao = SpringDaoFactory.getSeasonDao();			
			return seasonDao.create(seasonTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}	

	public SeasonTo findSeason(Long seasonId) throws InstanceNotFoundException, InternalErrorException {

		try {

			//SeasonDao seasonDao = SpringDaoFactory.getSeasonDao();
			return seasonDao.find(seasonId);

		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public void updateSeason(SeasonTo seasonTo) throws InternalErrorException {

		try{

			//SeasonDao seasonDao = SpringDaoFactory.getSeasonDao();
			seasonDao.update(seasonTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteSeason(SeasonTo seasonTo) throws SeasonTariffiedException, InternalErrorException {

		try{

			DeleteSeasonAction action = new DeleteSeasonAction(seasonTo, seasonDao);
			action.execute();

		} catch (SeasonTariffiedException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showSeasons() throws InternalErrorException {

		try{

			//SeasonDao seasonDao = SpringDaoFactory.getSeasonDao();
			return seasonDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public void createPrice(PriceTo priceTo) throws InternalErrorException {

		try{

			CreatePriceAction action = new CreatePriceAction(priceTo, boardDao, priceTypeDao, seasonDao, priceDao);			
			action.execute();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showSeasonPrices(Long season) throws InternalErrorException {

		try {

			//PriceDao priceDao = SpringDaoFactory.getPriceDao();
			return priceDao.findBySeason(season);

		} catch (InternalErrorException e) {
			throw e;
		}
	}

	public PriceTo findPrice(Long priceId, String board, String priceType, Long season) throws InstanceNotFoundException, InternalErrorException {

		try {

			//PriceDao priceDao = SpringDaoFactory.getPriceDao();
			return priceDao.find(priceId, board, priceType, season);

		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void updatePrice (PriceTo priceTo) throws InternalErrorException {

		try{

			//PriceDao priceDao = SpringDaoFactory.getPriceDao();
			priceDao.update(priceTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deletePrice (PriceTo priceTo) throws InternalErrorException {

		try{

			//PriceDao priceDao = SpringDaoFactory.getPriceDao();			
			priceDao.remove(priceTo);			

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public long maxPriceId() throws InternalErrorException {

		try{

			//PriceDao priceDao = SpringDaoFactory.getPriceDao();			
			return priceDao.maxId();			

		}catch (InternalErrorException e) {			
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public DayTo createDay(DayTo dayTo) throws InternalErrorException {

		try{

			//DayDao dayDao = SpringDaoFactory.getDayDao();				
			return dayDao.create(dayTo);


		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void updateDay(DayTo dayTo) throws InternalErrorException {

		try{

			//DayDao dayDao = SpringDaoFactory.getDayDao();			
			dayDao.update(dayTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteDaySeason(Calendar dayDate) throws SeasonTariffiedException, InstanceNotFoundException, InternalErrorException {

		try{

			DeleteDaySeasonAction action = new DeleteDaySeasonAction(dayDate, dayDao, seasonDao);
			action.execute();

		} catch (SeasonTariffiedException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void updateDaySeason(DayTo dayTo) throws InternalErrorException {

		try {

			UpdateDaySeasonAction action = new UpdateDaySeasonAction(dayDao, oweDao, conceptDao, requestDao, priceDao, dayTo);
			action.execute();	

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteDay(DayTo dayTo) throws InternalErrorException {

		try{

			//DayDao dayDao = SpringDaoFactory.getDayDao();
			dayDao.remove(dayTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showDays() throws InternalErrorException {

		try{

			//DayDao dayDao = SpringDaoFactory.getDayDao();
			return dayDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public Collection showSeasonDays(Long seasonId) throws InternalErrorException {
		try {
			return dayDao.findBySeason(seasonId);
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public ConceptTo createConcept(ConceptTo conceptTo) throws InternalErrorException {

		try{

			//ConceptDao conceptDao = SpringDaoFactory.getConceptDao();		
			return conceptDao.create(conceptTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}



	}

	public ConceptTo findConcept(String conceptCode) throws InstanceNotFoundException, InternalErrorException {

		try {

			//ConceptDao conceptDao = SpringDaoFactory.getConceptDao();
			return conceptDao.find(conceptCode);

		} catch (InstanceNotFoundException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void updateConcept(ConceptTo conceptTo) throws InternalErrorException {

		try{

			//ConceptDao conceptDao = SpringDaoFactory.getConceptDao();
			conceptDao.update(conceptTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteConcept(ConceptTo conceptTo) throws ConceptInUseException, InternalErrorException {

		try{

			DeleteConceptAction action = new DeleteConceptAction(conceptTo, conceptDao);
			action.execute();

		} catch (ConceptInUseException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showConcepts() throws InternalErrorException {

		try{

			//ConceptDao conceptDao = SpringDaoFactory.getConceptDao();
			return conceptDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public EmployeeTo createEmployee(EmployeeTo employeeTo) throws InternalErrorException {

		try{

			//EmployeeDao employeeDao = SpringDaoFactory.getEmployeeDao();		
			return employeeDao.create(employeeTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public EmployeeTo findEmployee(Long employeeId) throws InstanceNotFoundException, InternalErrorException {
		return employeeDao.find(employeeId);
	}

	public void updateEmployee(EmployeeTo employeeTo) throws InternalErrorException {

		try{

			//EmployeeDao employeeDao = SpringDaoFactory.getEmployeeDao();
			employeeDao.update(employeeTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public void deleteEmployee(EmployeeTo employeeTo) throws EmployeeWithWashesException, InternalErrorException {

		try{

			DeleteEmployeeAction action = new DeleteEmployeeAction(employeeTo, employeeDao);
			action.execute();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection showEmployees() throws InternalErrorException {

		try{

			//EmployeeDao employeeDao = SpringDaoFactory.getEmployeeDao();
			return employeeDao.findAll();

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}

	public UserTo registerUser(UserTo userTo) throws InternalErrorException {

		try {

			//UserDao userDao = SpringDaoFactory.getUserDao();
			return userDao.create(userTo);

		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		} 
	}

	public void changePassword(String login, String newPassword) throws InstanceNotFoundException, InternalErrorException {

		try {

			ChangePasswordAction action = new ChangePasswordAction(login,newPassword, userDao);
			action.execute();

		}catch (InstanceNotFoundException e) {
			throw e;
		}catch (InternalErrorException e) {
			throw e;
		}catch (Exception e) {
			throw new InternalErrorException(e);
		} 

	}

	public void removeUser(UserTo userTo) throws UserInUseException, InternalErrorException {

		try {

			RemoveUserAction action = new RemoveUserAction(userTo, userDao);
			action.execute();

		} catch (UserInUseException e) {
			throw e;
		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}
	}	

	public Collection usersList() throws InternalErrorException {

		try {

			//UserDao userDao = SpringDaoFactory.getUserDao();
			return userDao.findAll();

		} catch (InternalErrorException e) {
			throw e;
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

	public Collection rolesList() throws InternalErrorException {

		try {

			//RoleDao roleDao = SpringDaoFactory.getRoleDao();
			return roleDao.findAll();

		} catch (InternalErrorException e) {
			throw e;			
		} catch (Exception e) {
			throw new InternalErrorException(e);
		}

	}

}
