package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.address.dao.AddressDao;
import gz.cyberbrain.cyberhotel.model.bank.dao.BankDao;
import gz.cyberbrain.cyberhotel.model.card.dao.CardDao;
import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.client.to.AgencyTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.InvalidPayModeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAddressException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableBankException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCardException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

public class InsertAnAgencyAction {

	private AgencyTo agencyTo;
	private AddressDao addressDao;
	private BankDao bankDao;
	private CardDao cardDao;
	private ClientDao agencyDao;

	public InsertAnAgencyAction(AgencyTo agencyTo, AddressDao addressDao, BankDao bankDao, CardDao cardDao, ClientDao agencyDao) {
		this.agencyTo = agencyTo;
		this.addressDao = addressDao;
		this.bankDao = bankDao;
		this.cardDao = cardDao;
		this.agencyDao = agencyDao;		
	}

	public AgencyTo execute() 
	throws InvalidPayModeException, UnaceptableAddressException, 
	UnaceptableBankException, UnaceptableCardException,
	InternalErrorException {

		try {
			addressDao.find(agencyTo.getAddress());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableAddressException(agencyTo.getAddress());
		}

		try {
			bankDao.find(agencyTo.getBank());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableBankException(agencyTo.getBank());
		}

		try {
			cardDao.find(agencyTo.getCard());
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableCardException(agencyTo.getCard());
		}

		if (agencyTo.getPayMode() != "Contado"
				&& agencyTo.getPayMode() != "Tarxeta"
					&& agencyTo.getPayMode() != "Bono") throw new InvalidPayModeException(agencyTo.getPayMode()); 

		return (AgencyTo) agencyDao.create(agencyTo);


	}


}
