package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;

import gz.cyberbrain.cyberhotel.model.board.dao.BoardDao;
import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.pricetype.dao.PriceTypeDao;
import gz.cyberbrain.cyberhotel.model.request.dao.RequestDao;
import gz.cyberbrain.cyberhotel.model.request.to.RequestTo;
import gz.cyberbrain.cyberhotel.model.reservation.dao.ReservationDao;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAgencyException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableBoardException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCompanyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptablePriceTypeException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnavailableRoomException;
import gz.cyberbrain.cyberhotel.model.room.dao.RoomDao;
import gz.cyberbrain.cyberhotel.model.util.dates.Interval;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;

public class UpdateAReservationAction {

	private ReservationTo reservationTo;
	private Collection requestsTo;
	private BoardDao boardDao;
	private ClientDao agencyDao;  
	private ClientDao companyDao; 
	private ClientDao particularDao;
	private PriceTypeDao priceTypeDao;
	private RequestDao requestDao;
	private ReservationDao reservationDao;
	private RoomDao roomDao;

	public UpdateAReservationAction(ReservationTo reservationTo, Collection requestsTo, BoardDao boardDao, 
			ClientDao agencyDao,  ClientDao companyDao, ClientDao particularDao, PriceTypeDao priceTypeDao, 
			RequestDao requestDao, ReservationDao reservationDao, RoomDao roomDao) {
		this.reservationTo = reservationTo;
		this.requestsTo = requestsTo;
		this.boardDao = boardDao;
		this.agencyDao = agencyDao;  
		this.companyDao = companyDao; 
		this.particularDao = particularDao;
		this.priceTypeDao = priceTypeDao;
		this.requestDao = requestDao;
		this.reservationDao = reservationDao;
		this.roomDao = roomDao;
	}

	public void execute() 
	throws UnaceptableParticularException, UnaceptableBoardException, UnaceptablePriceTypeException, 
	UnaceptableAgencyException, UnaceptablePayerException, UnaceptableCompanyException, UnaceptableRoomException,
	UnavailableRoomException, InvalidIntervalException, InstanceNotFoundException, InternalErrorException {

		//ReservationDao reservationDao = SpringDaoFactory.getReservationDao();
		//RequestDao requestDao = SpringDaoFactory.getRequestDao();

		Collection oldRequests = reservationDao.findRequests(reservationTo.getReservationId());

		Iterator iter;

		Interval reservationInterval = new Interval(reservationTo.getIncomingDate(), reservationTo.getExpectedOutgoingDate());

		Calendar []dates = {reservationInterval.getStart(), reservationInterval.getEnd(), reservationInterval.getStart(), reservationInterval.getEnd()};

		iter = requestsTo.iterator();

		//RoomDao roomDao = SpringDaoFactory.getRoomDao();

		Collection existentRooms = roomDao.findAllIds();

		for (int i=0; iter.hasNext();i++){
			String currentRoomCode = ((RequestTo)iter.next()).getRoom();
			if(!existentRooms.contains(currentRoomCode)) throw new UnaceptableRoomException(currentRoomCode);
		}		

		//ClientDao particularDao = SpringDaoFactory.getParticularDao();	
		Collection existentParticulars = particularDao.findAllIds();

		iter = requestsTo.iterator();

		for (int i=0; iter.hasNext();i++){
			Long currentParticularId = ((RequestTo)iter.next()).getParticular();
			if(!existentParticulars.contains(currentParticularId)) throw new UnaceptableParticularException(currentParticularId);
		}

		//BoardDao boardDao = SpringDaoFactory.getBoardDao();
		Collection validBoards = boardDao.findAllCodes();		

		iter = requestsTo.iterator();

		for (int i=0; iter.hasNext();i++) {
			String currentBoardCode = ((RequestTo) iter.next()).getBoard();
			if(!validBoards.contains(currentBoardCode)) throw new UnaceptableBoardException(currentBoardCode);
		}

		//PriceTypeDao priceTypeDao = SpringDaoFactory.getPriceTypeDao();
		Collection validPriceTypes = priceTypeDao.findAllCodes();		

		iter = requestsTo.iterator();

		for (int i=0; iter.hasNext();i++) {
			String currentPriceTypeCode = ((RequestTo) iter.next()).getPriceType();
			if(!validPriceTypes.contains(currentPriceTypeCode)) throw new UnaceptablePriceTypeException(currentPriceTypeCode);
		}

		//ClientDao agencyDao = SpringDaoFactory.getAgencyDao();	
		Collection existentAgencies = agencyDao.findAllIds();

		Long currentAgencyId = reservationTo.getAgency();
		if ((currentAgencyId != null) && (!existentAgencies.contains(currentAgencyId))) throw new UnaceptableAgencyException(currentAgencyId);

		//ClientDao companyDao = SpringDaoFactory.getCompanyDao();
		Collection existentCompanies = companyDao.findAllIds();

		Long currentCompanyId = reservationTo.getCompany();
		if((currentCompanyId != null) && (!existentCompanies.contains(currentCompanyId))) throw new UnaceptableCompanyException(currentCompanyId);

		Long currentPayerId = reservationTo.getPayer();
		if (currentPayerId == null) throw new UnaceptablePayerException(currentPayerId);
		if(!existentParticulars.contains(currentPayerId)
				&& !existentAgencies.contains(currentPayerId)
				&& !existentCompanies.contains(currentPayerId)) throw new UnaceptablePayerException(currentPayerId);		

		reservationDao.update(reservationTo);

		iter = oldRequests.iterator();

		for (int i=0; iter.hasNext();i++) {
			RequestTo currentRequest = (RequestTo) iter.next();
			requestDao.remove(currentRequest);
		}

		reservationDao.findRequests(reservationTo.getReservationId());			

		Collection availableRooms = roomDao.freeBetweenDates(dates);		

		iter = requestsTo.iterator();

		for (int i=0; iter.hasNext();i++) {			
			String currentRoomCode = ((RequestTo) iter.next()).getRoom();
			if (!availableRooms.contains(currentRoomCode)) throw new UnavailableRoomException(currentRoomCode);

		}	

		RequestTo currentRequest;

		iter = requestsTo.iterator();

		for (int i=0; iter.hasNext();i++) {
			currentRequest = ((RequestTo) iter.next());
			currentRequest.setReservation(reservationTo.getReservationId());
			requestDao.create(currentRequest);
		}	

	}

}
