package gz.cyberbrain.cyberhotel.view.swing.edit;

import gz.cyberbrain.cyberhotel.controller.BoardController;
import gz.cyberbrain.cyberhotel.controller.PriceController;
import gz.cyberbrain.cyberhotel.controller.PriceTypeController;
import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.SeasonListAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.PricesTableModel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.NumberEditor;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TableItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel; 
import javax.swing.JScrollPane;
import javax.swing.JInternalFrame;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JButton;

public class PriceWindow extends JInternalFrame {

	private static final long serialVersionUID = 1L;

	private static PriceWindow priceWindow = null;

	private SaveAdapter saveAdapter = null;

	private Serializable id = null;

	private String dialogMessage;

	private String dialogTitle;

	private JPanel jContentPane = null;

	private JScrollPane seasonsScrollPane = null;

	private JList seasonsList = null;

	private JLabel seasonsLabel = null;

	private JScrollPane pricesScrollPane = null;

	private JTable pricesTable = null;

	private JLabel pricesLabel = null;

	private JButton saveButton = null;

	private JButton exitButton = null;

	private InfoPanel infoPanel = null;

	private JComboBox priceTypesComboBox = null;

	private JComboBox boardsComboBox = null;

	private JButton addButton = null;

	private JButton delButton = null;

	/**
	 * @param owner
	 */
	private PriceWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public static PriceWindow getInstance() {
		if (priceWindow == null) priceWindow = new PriceWindow();
		return priceWindow;
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}

	private JComboBox getPriceTypesComboBox() {
		if (priceTypesComboBox == null) {
			priceTypesComboBox = new JComboBox();
			priceTypesComboBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap priceType = new HashMap();					

					priceType = PriceTypeController.findPriceType((String) priceTypesComboBox.getSelectedItem());

					if (!inputErrors())
						getPricesTable().setValueAt(priceType.get("description"), getPricesTable().getSelectedRow(), 1);

				}
			});

		}
		return priceTypesComboBox;
	}

	private JComboBox getBoardsComboBox() {
		if (boardsComboBox == null) {
			boardsComboBox = new JComboBox();
			boardsComboBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					HashMap board = new HashMap();

					if (!inputErrors())
						board = BoardController.findBoard((String) boardsComboBox.getSelectedItem());					

					getPricesTable().setValueAt(board.get("description"), getPricesTable().getSelectedRow(), 3);

				}
			});

		}
		return boardsComboBox;
	}

	private void loadCombos() {

		try {

			Vector priceTypes = PriceTypeController.priceTypeList();
			Vector<Serializable>priceTypeIds = new Vector<Serializable>();

			Vector boards = BoardController.boardList();
			Vector<Serializable> boardIds = new Vector<Serializable>();

			for (int i=0; i< priceTypes.size(); i++) 
				priceTypeIds.addElement(((TextItem) priceTypes.elementAt(i)).getId());

			for (int i=0; i<boards.size(); i++)
				boardIds.addElement(((TextItem) boards.elementAt(i)).getId());

			getPriceTypesComboBox().setModel(new DefaultComboBoxModel(priceTypeIds));

			getBoardsComboBox().setModel(new DefaultComboBoxModel(boardIds));

			if (getPriceTypesComboBox().getItemCount() != 0) getPriceTypesComboBox().setSelectedIndex(0);
			if (getBoardsComboBox().getItemCount() != 0) getBoardsComboBox().setSelectedIndex(0);

		} catch (InternalErrorException e) {}

	}

	private Object [][] loadData() {

		Object [][] data = null;

		if (getSeasonsList().getModel().getSize() != 0) {

			try {

				Vector rows = PriceController.priceList((Long) ((TextItem) getSeasonsList().getSelectedValue()).getId());

				data = new Object[rows.size()][6];

				for (int i = 0; i<rows.size(); i++) {
					TableItem item = (TableItem) rows.elementAt(i);
					data[i][0] = (String) item.getColumn(0);
					data[i][1] = (String) item.getColumn(1);
					data[i][2] = (String) item.getColumn(2);
					data[i][3] = (String) item.getColumn(3);
					data[i][4] = (BigDecimal) item.getColumn(4);
					data[i][5] = ((Long) item.getId()[3]).longValue(); 
				}

			} catch (InstanceNotFoundException e) {
				e.printStackTrace();
			} catch (InternalErrorException e) {
				e.printStackTrace();
			}

		}

		return data;

	}

	private String[] loadTitles() {
		String [] titles = {
				Locale.get("PriceWindow.pricesTable.firstColumn.title"),
				Locale.get("PriceWindow.pricesTable.secondColumn.title"),
				Locale.get("PriceWindow.pricesTable.thirdColumn.title"),
				Locale.get("PriceWindow.pricesTable.fourthColumn.title"),
				Locale.get("PriceWindow.pricesTable.fifthColumn.title")
		};

		return titles;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
		this.setSize(798, 474);
		this.setLocation((dimension.width - this.getWidth())/2, (dimension.height - this.getHeight())/2);
		this.setResizable(false);
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		loadCombos();
		this.setContentPane(getJContentPane());		
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter() {

			public void internalFrameClosing(javax.swing.event.InternalFrameEvent evt) {

				(PriceWindow.getInstance()).clean();
				priceWindow = null;

			}

			public void internalFrameActivated(javax.swing.event.InternalFrameEvent evt) {

				initialize();

			}

		});
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			pricesLabel = new JLabel();
			pricesLabel.setBounds(new Rectangle(402, 61, 200, 19));
			pricesLabel.setText(Locale.get("PriceWindow.pricesLabel"));
			seasonsLabel = new JLabel();
			seasonsLabel.setBounds(new Rectangle(49, 61, 113, 19));
			seasonsLabel.setText(Locale.get("PriceWindow.seasonsLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getSeasonsScrollPane(), null);
			jContentPane.add(seasonsLabel, null);
			jContentPane.add(getPricesScrollPane(), null);
			jContentPane.add(pricesLabel, null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getExitButton(), null);
			jContentPane.add(getInfoPanel(),null);
			jContentPane.add(getAddButton(), null);
			jContentPane.add(getDelButton(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes seasonsScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getSeasonsScrollPane() {
		if (seasonsScrollPane == null) {
			seasonsScrollPane = new JScrollPane();
			seasonsScrollPane.setBounds(new Rectangle(16, 94, 180, 312));
			seasonsScrollPane.setViewportView(getSeasonsList());
		}
		return seasonsScrollPane;
	}

	/**
	 * This method initializes seasonsTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JList getSeasonsList() {
		if (seasonsList == null) {

			seasonsList = new JList();
			seasonsList.setListData((new SeasonListAdapter()).load());
			if (seasonsList.getComponentCount() != 0) seasonsList.setSelectedIndex(0);

			seasonsList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
			{
				public void valueChanged(javax.swing.event.ListSelectionEvent evt)
				{
					reloadTable();
				}
			});

		}
		return seasonsList;
	}

	/**
	 * This method initializes pricesScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getPricesScrollPane() {
		if (pricesScrollPane == null) {
			pricesScrollPane = new JScrollPane();
			pricesScrollPane.setBounds(new Rectangle(241, 93, 516, 310));
			pricesScrollPane.setViewportView(getPricesTable());
		}
		return pricesScrollPane;
	}

	/**
	 * This method initializes pricesTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getPricesTable() {
		if (pricesTable == null) {

			pricesTable = new JTable() {

				private static final long serialVersionUID = 1L;

				public boolean isCellEditable(int row, int col) {
					return col == 1 || col == 3 ? false : true;
				}								

			};

			pricesTable.setModel(new PricesTableModel(loadData(), loadTitles()));			
			if (pricesTable.getRowCount() > 0 ) pricesTable.setRowSelectionInterval(0, 0);			
			pricesTable.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(getPriceTypesComboBox()));
			pricesTable.getColumnModel().getColumn(2).setCellEditor(new DefaultCellEditor(getBoardsComboBox()));			
			pricesTable.getColumnModel().getColumn(4).setCellEditor(new NumberEditor());

			pricesTable.getModel().addTableModelListener(new javax.swing.event.TableModelListener() {

				public void tableChanged(javax.swing.event.TableModelEvent e) {}

			});

		}
		return pricesTable;
	}

	private void reloadTable() {

		pricesTable.setModel(new PricesTableModel(loadData(), loadTitles()));			
		if (pricesTable.getRowCount() > 0 ) pricesTable.setRowSelectionInterval(0, 0);			
		pricesTable.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(getPriceTypesComboBox()));
		pricesTable.getColumnModel().getColumn(2).setCellEditor(new DefaultCellEditor(getBoardsComboBox()));			
		pricesTable.getColumnModel().getColumn(4).setCellEditor(new NumberEditor());

		pricesTable.repaint();

	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setBounds(new Rectangle(591, 414, 76, 16));
			saveButton.setText(Locale.get("PriceWindow.saveButton"));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					boolean saveOk = true;

					for (int i = 0; i < getPricesTable().getModel().getRowCount(); i++) {

						PricesTableModel tableModel = (PricesTableModel) getPricesTable().getModel();

						if (i > tableModel.getDataRowCount() - 1 && checkRowValues(i)) {					

							Object[] row = {
									(String) getPricesTable().getValueAt(i, 0),
									(String) getPricesTable().getValueAt(i, 1),
									(String) getPricesTable().getValueAt(i, 2),
									(String) getPricesTable().getValueAt(i, 3),
									new BigDecimal(getPricesTable().getValueAt(i, 4).toString()),
									new Long(1)
							};

							tableModel.addDataRow(row);
							getPricesTable().setModel(tableModel);

							Long season = (Long)((TextItem) getSeasonsList().getSelectedValue()).getId();
							String priceType = (String) getPricesTable().getValueAt(i, 0);						
							String board = (String) getPricesTable().getValueAt(i, 2);
							BigDecimal value = new BigDecimal(getPricesTable().getValueAt(i, 4).toString());				
							Long priceId = (Long) ((PricesTableModel) getPricesTable().getModel()).getCustomValueAt(i, 5);

							try {
								PriceController.findPrice(season, priceType, board, priceId);							
							} catch (InstanceNotFoundException e1) {
								try {
									PriceController.createPrice(season, priceType, board, priceId, value);
								} catch (InternalErrorException e2) {}
							} catch (InternalErrorException e1) {}

							try {						
								PriceController.updatePrice(season, priceType, board, priceId, value);
							} catch (InternalErrorException e1) {}																	

						} else {
							Long season = (Long)((TextItem) getSeasonsList().getSelectedValue()).getId();
							String priceType = (String) getPricesTable().getValueAt(i, 0);						
							String board = (String) getPricesTable().getValueAt(i, 2);
							BigDecimal value = new BigDecimal(getPricesTable().getValueAt(i, 4).toString());				
							Long priceId = (Long) ((PricesTableModel) getPricesTable().getModel()).getCustomValueAt(i, 5);
							try {
								PriceController.updatePrice(season, priceType, board, priceId, value);
							} catch (InternalErrorException e1) {}
						}

						if (!checkRowValues(i)) {
							setDialogTitle(Locale.get("JOptionPane.PriceWindow.saveButton.title.wrong"));
							setDialogMessage(Locale.get("JOptionPane.PriceWindow.saveButton.message.wrong"));

							Object[] options = {Locale.get("JOptionPane.Ok")};
							JOptionPane.showOptionDialog(RoomTypeWindow.getInstance(),
									getDialogMessage(),
									getDialogTitle(),
									JOptionPane.OK_OPTION,
									JOptionPane.INFORMATION_MESSAGE,
									null,
									options,
									options[0]);
							saveOk = false;
							break;
						}

					}

					if (saveOk) {
						setDialogTitle(Locale.get("JOptionPane.PriceWindow.saveButton.title.ok"));
						setDialogMessage(Locale.get("JOptionPane.PriceWindow.saveButton.message.ok"));

						Object[] options = {Locale.get("JOptionPane.Ok")};
						JOptionPane.showOptionDialog(RoomTypeWindow.getInstance(),
								getDialogMessage(),
								getDialogTitle(),
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);						
					}

				}

			});

		}
		return saveButton;
	}

	/**
	 * This method initializes exitButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getExitButton() {
		if (exitButton == null) {
			exitButton = new JButton();
			exitButton.setBounds(new Rectangle(680, 414, 76, 16));
			exitButton.setText(Locale.get("PriceWindow.exitButton"));
			exitButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					(PriceWindow.getInstance()).dispose();
					priceWindow = null;

				}
			});

		}
		return exitButton;
	}

	/**
	 * This method initializes infoPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(0,0,this.getWidth(),50);
		}
		return infoPanel;
	}


	public void setInfoPanel(InfoPanel ipanel) {		  

		jContentPane.remove(infoPanel);
		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	private void cleanTable() {


		getPricesTable().setModel(new PricesTableModel(loadData(), loadTitles()));
		if (getPricesTable().getRowCount() > 0 ) getPricesTable().setRowSelectionInterval(0, 0);			
		getPricesTable().getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(getPriceTypesComboBox()));
		getPricesTable().getColumnModel().getColumn(2).setCellEditor(new DefaultCellEditor(getBoardsComboBox()));			
		getPricesTable().getColumnModel().getColumn(4).setCellEditor(new NumberEditor());

		getPricesTable().getModel().addTableModelListener(new javax.swing.event.TableModelListener() {

			public void tableChanged(javax.swing.event.TableModelEvent e) {}

		});

	}


	public void clean() {
		cleanTable();
	}

	/**
	 * This method initializes addButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton();
			addButton.setBounds(new Rectangle(244, 414, 76, 16));
			addButton.setText(Locale.get("PriceWindow.addButton"));
			addButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					PricesTableModel tableModel = (PricesTableModel) getPricesTable().getModel();					
					tableModel.addRow(new Vector());					
					getPricesTable().setRowSelectionInterval(getPricesTable().getRowCount() - 1, getPricesTable().getRowCount() - 1);					

				}
			});
		}
		return addButton;
	}

	/**
	 * This method initializes delButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getDelButton() {
		if (delButton == null) {
			delButton = new JButton();
			delButton.setBounds(new Rectangle(333, 414, 76, 16));
			delButton.setText(Locale.get("PriceWindow.delButton"));
			delButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					setDialogMessage(Locale.get("JOptionPane.PriceWindow.delButton.message"));
					setDialogTitle(Locale.get("JOptionPane.PriceWindow.delButton.title"));					

					Object[] options = {Locale.get("JOptionPane.Yes"), Locale.get("JOptionPane.No")};
					int n = JOptionPane.showOptionDialog(RoomTypeWindow.getInstance(),
							getDialogMessage(),
							getDialogTitle(),
							JOptionPane.YES_NO_OPTION,
							JOptionPane.INFORMATION_MESSAGE,
							null,
							options,
							options[0]);

					if (n == JOptionPane.YES_OPTION && checkRowValues(getPricesTable().getSelectedRow())) {												

						Long season = (Long)((TextItem) getSeasonsList().getSelectedValue()).getId();
						String priceType = (String) getPricesTable().getValueAt(getPricesTable().getSelectedRow(), 0);						
						String board = (String) getPricesTable().getValueAt(getPricesTable().getSelectedRow(), 2);				
						Long priceId = (Long) ((PricesTableModel) getPricesTable().getModel()).getCustomValueAt(getPricesTable().getSelectedRow(), 5);

						try {
							PriceController.deletePrice(season, priceType, board, priceId);
						} catch (InstanceNotFoundException e1) {
						} catch (InternalErrorException e1) {}

						PricesTableModel tableModel = (PricesTableModel) getPricesTable().getModel();
						tableModel.removeDataRow(getPricesTable().getSelectedRow());
						tableModel.removeRow(getPricesTable().getSelectedRow());						
						getPricesTable().setModel(tableModel);

						if (getPricesTable().getRowCount() != 0) getPricesTable().setRowSelectionInterval(0, 0);

					}

					if (n == JOptionPane.YES_OPTION && !checkRowValues(getPricesTable().getSelectedRow())) {
						PricesTableModel tableModel = (PricesTableModel) getPricesTable().getModel();
						tableModel.removeRow(getPricesTable().getSelectedRow());						
						getPricesTable().setModel(tableModel);
						if (getPricesTable().getRowCount() != 0) getPricesTable().setRowSelectionInterval(getPricesTable().getRowCount()-1, getPricesTable().getRowCount()-1);
					}

				}
			});
		}
		return delButton;
	}

	private boolean checkRowValues(int row) {

		boolean validPriceType = false;
		boolean validBoard = false;
		boolean validValue = false;
		boolean validRow = false;

		try {

			Vector priceTypes = PriceTypeController.priceTypeList();
			Vector boards = BoardController.boardList();

			for (int i=0; i<priceTypes.size();i++) 				{
				if (((TextItem)priceTypes.elementAt(i)).getId().equals( (String) getPricesTable().getValueAt(row, 0) )) { 
					validPriceType = true;
					break;
				}
			}

			for (int i=0; i<boards.size();i++) {
				if (((TextItem)boards.elementAt(i)).getId() .equals( (String) getPricesTable().getValueAt(row, 2) )) { 
					validBoard = true;
					break;
				}

			}			

			try {
				new BigDecimal((String) getPricesTable().getValueAt(row, 4).toString());
				validValue = true;
			} catch (NullPointerException e) {
				validValue = false;
			}	

			validRow = validPriceType && validBoard && validValue;

		} catch (InternalErrorException e) {}

		return validRow;

	}

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty()) {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("instanceNotFound")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("PriceWindow.errors.instanceNotFound")+ "</html>");			

				Toolkit.getDefaultToolkit().beep();

				return true;
			}

		}

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="8,10"
