package gz.cyberbrain.cyberhotel.view.swing.util.gui;

import gz.cyberbrain.cyberhotel.controller.RoomController;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.view.swing.reception.RoomPlanningWindow;
import gz.cyberbrain.cyberhotel.view.swing.reception.WashWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.Image;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;

import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

public class RoomPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	private String roomCode = null;
	private JPanel bedPanel = null;
	private JPanel smokingPanel = null;
	private JPanel healthPanel = null;
	private JPanel trashPanel = null;
	private JPanel voidPanel = null;
	private String imgPath = PathHandler.getImagesPath();
	private JPanel firstPanel = null;  //  @jve:decl-index=0:visual-constraint="314,134"
	private JPanel numPanel = null;  //  @jve:decl-index=0:visual-constraint="62,121"
	private JLabel numLabel = null;
	private boolean smokers = false;
	private boolean clean = true;
	private boolean infected = false;
	private String occupancyStatus = null;
	private JPopupMenu roomMenu = null;
	private JLabel backLabel = null;	

	/**
	 * This is the default constructor
	 */
	public RoomPanel() {
		super();
		initialize();
	}

	public void setRoomCode(String roomCode) {
		this.roomCode = roomCode;
	}
	
	public String getRoomCode() {
		return roomCode;
	}
	
	public JLabel getBackLabel() {
		return backLabel;
	}

	public void setBackLabel(JLabel backLabel) {
		this.backLabel = backLabel;
		this.backLabel.setBounds(1, 1, this.getWidth()-2, this.getHeight()-2);
	}
	
	public void repaintBackground() {
		this.removeAll();
		initialize();
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setLayout(null);
		this.setSize(89, 120);
		this.add(getNumPanel(), null);
		this.add(getFirstPanel(), null);		
		this.setBorder(new LineBorder(Color.BLACK, 1));
		this.add(getRoomMenu());
		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
		this.addMouseListener(new MouseAdapter() {

			/*public void mouseClicked(MouseEvent ae){

			}*/

			public void mouseEntered(MouseEvent p0)
			{					
				setBorder(new LineBorder(Color.GREEN, 5));
				getNumPanel().setBounds(5,5,getWidth()-10,10);
				getNumLabel().setBounds(0,0,getWidth()-10,10);
				getBackLabel().setBounds(5, 5, getWidth() - 10, getHeight() - 10);
			}

			public void mouseExited(MouseEvent p0)
			{					
				setBorder(new LineBorder(Color.BLACK,3));
				getNumPanel().setBounds(3, 3, getWidth()-6,14);
				getNumLabel().setBounds(0,0,getWidth()-6,14);
				getBackLabel().setBounds(1, 1, getWidth()-2, getHeight()-1);
			}
		});

	}

	private JPopupMenu getRoomMenu() {
		if (roomMenu == null) {
			
			roomMenu = new JPopupMenu();
			
			JMenuItem lockItem = new JMenuItem(Locale.get("RoomPanel.lockItem"));
			lockItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					RoomController.lock(getRoomCode());		
					if (getOccupancyStatus().equals("Libre")) {
						remove(getBackLabel());
						setOccupancyStatus("Bloqueada");
						repaint();		
						ErrorsHandler.clear();
						(RoomPlanningWindow.getInstance()).cleanErrors();
					}
					
					(RoomPlanningWindow.getInstance()).showErrors();
					
				}
			});
			
			JMenuItem unlockItem = new JMenuItem(Locale.get("RoomPanel.unlockItem"));
			unlockItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					RoomController.unlock(getRoomCode());					
					if (getOccupancyStatus().equals("Bloqueada")) {
						remove(getBackLabel());
						setOccupancyStatus("Libre");
						repaint();
					}
					ErrorsHandler.clear();
					(RoomPlanningWindow.getInstance()).cleanErrors();					
				}
			});
			
			JMenuItem cleanItem = new JMenuItem(Locale.get("RoomPanel.cleanItem"));
			cleanItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					WashWindow washWindow = WashWindow.getInstance();
					washWindow.setRoomCode(getRoomCode());
					washWindow.setVisible(true);
					repaint();
					
					
				}
			});
			
			JMenuItem healthItem = new JMenuItem(Locale.get("RoomPanel.healthItem"));
			roomMenu.add(lockItem);
			roomMenu.add(unlockItem);
			roomMenu.add(cleanItem);
			roomMenu.add(healthItem);
		}
		return roomMenu;
	}

	/**
	 * This method initializes bedPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getBedPanel() {
		if (bedPanel == null) {
			bedPanel = new JPanel();
			bedPanel.setLayout(new GridBagLayout());
			bedPanel.setBounds(new Rectangle(0, (firstPanel.getHeight()/3) * 2, firstPanel.getWidth(), firstPanel.getHeight()/3));
			bedPanel.setOpaque(false);
		}
		return bedPanel;
	}

	/**
	 * This method initializes smokingPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getSmokingPanel() {
		if (smokingPanel == null) {
			smokingPanel = new JPanel();
			smokingPanel.setLayout(new GridBagLayout());
			smokingPanel.setBounds(new Rectangle(firstPanel.getWidth()/2, 0, firstPanel.getWidth()/2, firstPanel.getHeight()/3));
			smokingPanel.setOpaque(false);
		}
		return smokingPanel;
	}

	/**
	 * This method initializes healthPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getHealthPanel() {
		if (healthPanel == null) {
			healthPanel = new JPanel();
			healthPanel.setLayout(new GridBagLayout());
			healthPanel.setBounds(new Rectangle(firstPanel.getWidth()/2, firstPanel.getHeight()/3, firstPanel.getWidth()/2, firstPanel.getHeight()/3));
			healthPanel.setOpaque(false);
		}
		return healthPanel;
	}

	/**
	 * This method initializes trashPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getTrashPanel() {
		if (trashPanel == null) {
			trashPanel = new JPanel();
			trashPanel.setLayout(new GridBagLayout());
			trashPanel.setBounds(new Rectangle(0, firstPanel.getHeight()/3, firstPanel.getWidth()/2, firstPanel.getHeight()/3));
			trashPanel.setOpaque(false);
		}
		return trashPanel;
	}

	/**
	 * This method initializes voidPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getVoidPanel() {
		if (voidPanel == null) {
			voidPanel = new JPanel();
			voidPanel.setLayout(new GridBagLayout());
			voidPanel.setBounds(new Rectangle(0, 0, firstPanel.getWidth()/2, firstPanel.getHeight()/3));
			voidPanel.setOpaque(false);			
		}
		return voidPanel;
	}

	protected void processMouseEvent(MouseEvent e){
		if(e.isPopupTrigger())
			getRoomMenu().show(e.getComponent(),e.getX(),e.getY());
		else
			super.processMouseEvent(e);
	}

	private ImageIcon getSmokingIcon() {
		ImageIcon smoking;
		if (!smokers) smoking = new ImageIcon(imgPath + "nosmoking.png");
		else smoking  = new ImageIcon(imgPath + "smoking.png");
		return new ImageIcon(smoking.getImage().
				getScaledInstance(getSmokingPanel().getWidth()-10, getSmokingPanel().getHeight()-10, Image.SCALE_SMOOTH));
	}

	private ImageIcon getTrashIcon() {
		ImageIcon trash = new ImageIcon(imgPath + "vacuum.png"); 
		return new ImageIcon(trash.getImage().
				getScaledInstance(getTrashPanel().getWidth()-10, getSmokingPanel().getHeight()-10, Image.SCALE_SMOOTH));
	}

	private ImageIcon getHealthIcon() {
		ImageIcon health = new ImageIcon(imgPath + "biohazard.png"); 
		return new ImageIcon(health.getImage().
				getScaledInstance(getHealthPanel().getWidth()-10, getSmokingPanel().getHeight()-10, Image.SCALE_SMOOTH));
	}

	private ImageIcon getBedIcon() {
		ImageIcon bed = new ImageIcon(imgPath + "cama.png"); 
		return new ImageIcon(bed.getImage().
				getScaledInstance(getBedPanel().getWidth()-10, getSmokingPanel().getHeight()-10, Image.SCALE_SMOOTH));
	}

	public void showStatus() {
		getSmokingPanel().add(new JLabel(getSmokingIcon()));
		if (!clean) getTrashPanel().add(new JLabel(getTrashIcon()));
		if (infected) getHealthPanel().add(new JLabel(getHealthIcon()));
		getBedPanel().add(new JLabel(getBedIcon()));
	}

	/**
	 * This method initializes firstPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getFirstPanel() {
		if (firstPanel == null) {
			firstPanel = new JPanel();
			firstPanel.setLayout(null);
			firstPanel.setBounds(0,15, 89, 105);
			firstPanel.add(getVoidPanel(), null);
			firstPanel.add(getSmokingPanel(), null);
			firstPanel.add(getTrashPanel(), null);
			firstPanel.add(getHealthPanel(), null);		
			firstPanel.add(getBedPanel(), null);
			firstPanel.setOpaque(false);
		}
		return firstPanel;
	}

	/**
	 * This method initializes numPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getNumPanel() {
		if (numPanel == null) {
			numPanel = new JPanel();
			numPanel.setLayout(new GridBagLayout());
			numPanel.setBounds(3,3, this.getWidth()-6,14);		
			numPanel.add(getNumLabel(), null);
			numPanel.setBackground(Color.BLUE);			
		}
		return numPanel;
	}

	private JLabel getNumLabel() {
		if (numLabel == null) {
			numLabel = new JLabel("",SwingConstants.CENTER);
			numLabel.setBounds(0,0,this.getWidth()-4,14);			
			numLabel.setForeground(Color.WHITE);
		}
		return numLabel;
	}

	public void setNumber(String number) {
		getNumLabel().setText("<html><font> <b>" + number + "</b></font></html>");
	}

	public void setSmokers(boolean smokers) {
		this.smokers = smokers;
	}

	public void setClean(boolean clean) {
		this.clean = clean;
	}

	public void setInfected(boolean infected) {
		this.infected = infected;
	}

	public void setOccupancyStatus(String occupancyStatus) {
				
		this.occupancyStatus = occupancyStatus;
		
		if (occupancyStatus.equals("Libre")) {
			this.setBackLabel(new JLabel(new ImageIcon(imgPath + "free.png")));
			this.add(getBackLabel());
		}
		if (occupancyStatus.equals("Ocupada")) {
			this.setBackLabel(new JLabel(new ImageIcon(imgPath + "occupated.png")));
			this.add(getBackLabel());				
		}
		if (occupancyStatus.equals("Bloqueada")) {
			this.setBackLabel(new JLabel(new ImageIcon(imgPath + "locked.png")));
			this.add(getBackLabel());
		}
		
	}

	public String getNumber() {
		return getNumLabel().getText();
	}

	public boolean isSmokers() {
		return smokers;
	}

	public boolean isClean() {
		return clean;
	}

	public boolean isInfected() {
		return infected;
	}

	public String getOccupancyStatus() {
		return occupancyStatus;
	}	

}  //  @jve:decl-index=0:visual-constraint="214,120"
